import copy
import logging
from unittest import mock

from fedora_messaging.api import Message

from resultsdb_listener import consumer


valid_message_dict = {
    "topic": "org.centos.prod.ci.koji-build.test.complete",
    "body": {
        "artifact": {
            "component": "rust-miniz_oxide_c_api",
            "dependencies": [],
            "id": 37704421,
            "issuer": "ignatenkobrain",
            "nvr": "rust-miniz_oxide_c_api-0.2.3-1.fc32",
            "scratch": False,
            "source": "git+https://src.fedoraproject.org/rpms/"
            "rust-miniz_oxide_c_api.git#"
            "e3e54e569d1b74166c006b9046c49c2815d26f54",
            "type": "koji-build",
        },
        "contact": {
            "docs": "https://pagure.io/standard-test-roles",
            "email": "ci@lists.fedoraproject.org",
            "irc": "#fedora-ci",
            "name": "fedora-ci",
            "team": "fedora-ci",
            "url": "https://jenkins-continuous-infra.apps.ci.centos.org/",
        },
        "generated_at": "2019-09-17T13:49:02.649Z",
        "pipeline": {
            "build": "6371",
            "id": "675bde0e-e235-4d33-9bed-6e1e0a5d8d18",
            "name": "fedora-rawhide-build-pipeline",
            "stage": {"name": "package-tests"},
        },
        "run": {
            "log": "https://jenkins-continuous-infra.apps.ci.centos.org/"
            "job/fedora-rawhide-build-pipeline/6371/console",
            "rebuild": "https://jenkins-continuous-infra.apps.ci.centos.org/"
            "job/fedora-rawhide-build-pipeline/6371/rebuild/parameterized",
            "url": "https://jenkins-continuous-infra.apps.ci.centos.org/"
            "blue/organizations/jenkins/fedora-rawhide-build-pipeline/"
            "detail/fedora-rawhide-build-pipeline/6371/pipeline/",
        },
        "test": {
            "category": "functional",
            "namespace": "fedora-ci.koji-build",
            "result": "failed",
            "type": "tier0",
        },
        "version": "0.2.1",
    },
}


class TestPipelineInvalidArtifact(object):
    @mock.patch("resultsdb_listener.consumer.senders")
    def test_parse_invalid_artifact_message(self, senders, caplog):
        caplog.set_level(logging.INFO)
        processor = mock.MagicMock()
        senders.get_sender.return_value = processor
        cons = consumer.Consumer()

        invalid_artifact_message_dict = copy.deepcopy(valid_message_dict)
        del invalid_artifact_message_dict["body"]["artifact"]["component"]
        invalid_artifact_message = Message(
            topic=invalid_artifact_message_dict["topic"],
            body=invalid_artifact_message_dict["body"],
        )
        invalid_artifact_message.id = "random_uuid"

        cons(invalid_artifact_message)

        assert processor().send.call_count == 0
        processor().send.assert_not_called()
        assert (
            caplog.records[-2].message
            == "Ignoring message format, missing component in 'artifact' "
            "- msg_id : random_uuid"
        )


class TestPipelineInvalidRun(object):
    @mock.patch("resultsdb_listener.consumer.senders")
    def test_parse_invalid_run_message(self, senders, caplog):
        caplog.set_level(logging.INFO)
        processor = mock.MagicMock()
        senders.get_sender.return_value = processor
        cons = consumer.Consumer()

        invalid_run_message_dict = copy.deepcopy(valid_message_dict)
        del invalid_run_message_dict["body"]["run"]["url"]
        invalid_run_message = Message(
            topic=invalid_run_message_dict["topic"],
            body=invalid_run_message_dict["body"],
        )
        invalid_run_message.id = "random_uuid"

        cons(invalid_run_message)

        assert processor().send.call_count == 0
        processor().send.assert_not_called()
        assert (
            caplog.records[-2].message
            == "Ignoring message format, missing url in 'run' "
            "- msg_id : random_uuid"
        )


class TestPipelineInvalidTest(object):
    @mock.patch("resultsdb_listener.consumer.senders")
    def test_parse_invalid_test_message(self, senders, caplog):
        caplog.set_level(logging.INFO)
        processor = mock.MagicMock()
        senders.get_sender.return_value = processor
        cons = consumer.Consumer()

        invalid_test_message_dict = copy.deepcopy(valid_message_dict)
        del invalid_test_message_dict["body"]["test"]["category"]
        invalid_test_message = Message(
            topic=invalid_test_message_dict["topic"],
            body=invalid_test_message_dict["body"],
        )
        invalid_test_message.id = "random_uuid"

        cons(invalid_test_message)

        assert processor().send.call_count == 0
        processor().send.assert_not_called()
        assert (
            caplog.records[-2].message
            == "Ignoring message format, missing category in 'test' "
            "- msg_id : random_uuid"
        )
